#include "stdafx.h"
#include "UIC_IDirect3DTexture9.h"
#include "UIC_IDirect3DDevice9Ex.h"

UIC_IDirect3DTexture9::UIC_IDirect3DTexture9(IDirect3DTexture9 *pTex, UIC_IDirect3DDevice9Ex* pDevice)
	: myOriginal(pTex)
	, myDevice(pDevice)
	, mySetOnBindFlags(0)
	, mySetTempFlags(0)
	, myClearTempFlags(0)
	, myIsInitialized(false)
	, myIsLockedForInit(false)
	, myIsAddedToList(false)
	, mySuppressDrawingWhenActiveFlags(0)
	, myWidth(0)
	, myHeight(0)
	, myLevels(0)
	, myFormat(0)
{
}

UIC_IDirect3DTexture9::~UIC_IDirect3DTexture9()
{
}

HRESULT APIENTRY UIC_IDirect3DTexture9::QueryInterface(REFIID riid, void** ppvObj)
{
	HRESULT hRes = myOriginal->QueryInterface(riid, ppvObj);
	if (*ppvObj == myOriginal)
		*ppvObj = this;
	return hRes;
}

ULONG APIENTRY UIC_IDirect3DTexture9::AddRef()
{
	return myOriginal->AddRef();
}

ULONG APIENTRY UIC_IDirect3DTexture9::Release()
{
	ULONG count = myOriginal->Release();
	if (!count)
	{
		if (myIsAddedToList)
			myDevice->myState.RemoveTexture(this);
		delete this;
	}
	return count;
}

HRESULT APIENTRY UIC_IDirect3DTexture9::GetDevice(IDirect3DDevice9** ppDevice)
{
	*ppDevice = myDevice;
	return D3D_OK;
}

//this function yields for the non switched texture object
HRESULT APIENTRY UIC_IDirect3DTexture9::SetPrivateData(REFGUID refguid, CONST void* pData, DWORD SizeOfData, DWORD Flags)
{
	return myOriginal->SetPrivateData(refguid, pData, SizeOfData, Flags);
}

//this function yields for the non switched texture object
HRESULT APIENTRY UIC_IDirect3DTexture9::GetPrivateData(REFGUID refguid, void* pData, DWORD* pSizeOfData)
{
	return myOriginal->GetPrivateData(refguid, pData, pSizeOfData);
}

//this function yields for the non switched texture object
HRESULT APIENTRY UIC_IDirect3DTexture9::FreePrivateData(REFGUID refguid)
{
	return myOriginal->FreePrivateData(refguid);
}

DWORD APIENTRY UIC_IDirect3DTexture9::SetPriority(DWORD PriorityNew)
{
	return myOriginal->SetPriority(PriorityNew);
}

DWORD APIENTRY UIC_IDirect3DTexture9::GetPriority()
{
	return myOriginal->GetPriority();
}

void APIENTRY UIC_IDirect3DTexture9::PreLoad()
{
	myOriginal->PreLoad();
}

D3DRESOURCETYPE APIENTRY UIC_IDirect3DTexture9::GetType()
{
	return myOriginal->GetType();
}

DWORD APIENTRY UIC_IDirect3DTexture9::SetLOD(DWORD LODNew)
{
	return myOriginal->SetLOD(LODNew);
}

DWORD APIENTRY UIC_IDirect3DTexture9::GetLOD()
{
	return myOriginal->GetLOD();
}

DWORD APIENTRY UIC_IDirect3DTexture9::GetLevelCount()
{
	return myOriginal->GetLevelCount();
}

HRESULT APIENTRY UIC_IDirect3DTexture9::SetAutoGenFilterType(D3DTEXTUREFILTERTYPE FilterType)
{
	return myOriginal->SetAutoGenFilterType(FilterType);
}

D3DTEXTUREFILTERTYPE APIENTRY UIC_IDirect3DTexture9::GetAutoGenFilterType()
{
	return myOriginal->GetAutoGenFilterType();
}

void APIENTRY UIC_IDirect3DTexture9::GenerateMipSubLevels()
{
	myOriginal->GenerateMipSubLevels();
}

HRESULT APIENTRY UIC_IDirect3DTexture9::GetLevelDesc(UINT Level, D3DSURFACE_DESC *pDesc)
{
	return myOriginal->GetLevelDesc(Level, pDesc);
}

HRESULT APIENTRY UIC_IDirect3DTexture9::GetSurfaceLevel(UINT Level, IDirect3DSurface9** ppSurfaceLevel)
{
	return myOriginal->GetSurfaceLevel(Level, ppSurfaceLevel);
}

HRESULT APIENTRY UIC_IDirect3DTexture9::LockRect(UINT Level, D3DLOCKED_RECT* pLockedRect, CONST RECT* pRect, DWORD Flags)
{
	HRESULT ret = myOriginal->LockRect(Level, pLockedRect, pRect, Flags);
	if (ret == D3D_OK)
	{
		if (!myIsInitialized && !(Flags & D3DLOCK_READONLY) && pLockedRect->pBits && myLevels == 1 && myFormat == D3DFMT_DXT5)
		{
			myIsLockedForInit = true;
			myLockedRect = *pLockedRect;
		}
	}
	return ret;
}

static uint32_t crc_table[256];
 
static void make_crc_table()
{
    for (uint32_t i = 0; i < 256; i++) {
        uint32_t c = i;
        for (int j = 0; j < 8; j++) {
            c = (c & 1) ? (0xEDB88320 ^ (c >> 1)) : (c >> 1);
        }
        crc_table[i] = c;
    }
}
 
static uint32_t crc32(const uint8_t *buf, size_t len)
{
    uint32_t c = 0xFFFFFFFF;
    for (size_t i = 0; i < len; i++) {
        c = crc_table[(c ^ buf[i]) & 0xFF] ^ (c >> 8);
    }
    return c ^ 0xFFFFFFFF;
}

HRESULT APIENTRY UIC_IDirect3DTexture9::UnlockRect(UINT Level)
{
	if (myIsLockedForInit)
	{
		myIsLockedForInit = false;
		myIsInitialized = true;

		unsigned size = myWidth * myHeight * 1;

		if (!crc_table[255])
			make_crc_table();

		myCRC32 = crc32((const uint8_t*) myLockedRect.pBits, size);

		myDevice->myState.AddTexture(this);
	}

	return myOriginal->UnlockRect(Level);
}

HRESULT APIENTRY UIC_IDirect3DTexture9::AddDirtyRect(CONST RECT* pDirtyRect)
{
	return myOriginal->AddDirtyRect(pDirtyRect);
}
