#pragma once

struct UIC_SharedMemory
{
	static const unsigned MaxKnownTextures = 64;

	enum Flags : unsigned {
		CombatPossible			= 0x01,
		UIOpen					= 0x02,
		InCombat				= 0x04,
		ChatOpen				= 0x08,

		CombatModeWanted		= 0x10,
		CombatModeActive		= 0x20,
		Always					= 0x80,

		ActiveCheckMask			= CombatPossible|UIOpen|CombatModeWanted,
		ActiveCheckTest			= CombatPossible|CombatModeWanted,

		BlockAllRendering		= 0x800,
	};

	enum Command : unsigned {
		Nothing,
		ClearSelection,
		SelectLastTexture,
		SelectNextTexture,
		SelectPreviousTexture,
		SaveCurrentTextureInfo,
	};

	struct KnownSize {
		unsigned short myWidth;
		unsigned short myHeight;
	};

	struct TextureInfo {
		unsigned myWidth;
		unsigned myHeight;
		unsigned myCRC32;
		unsigned mySetOnBindFlags;
		unsigned mySuppressDrawingWhenActiveFlags;
		unsigned mySetTempFlags;
		unsigned myClearTempFlags;

		TextureInfo(unsigned w, unsigned h, unsigned crc, unsigned flags, unsigned suppress, unsigned setTemp, unsigned clearTemp)
			: myWidth(w), myHeight(h), myCRC32(crc), mySetOnBindFlags(flags), mySuppressDrawingWhenActiveFlags(suppress), mySetTempFlags(setTemp), myClearTempFlags(clearTemp) {}
	};

	volatile Command myCommand;
	volatile unsigned myFlagsFromGame;
	volatile unsigned myFlagsFromInput;
	volatile unsigned myFlags;
	volatile unsigned myFrameCounter;
	volatile unsigned myHasFocus;

	bool myIsDevMode;

	unsigned myNumKnownSizes;
	KnownSize myKnownSizes[MaxKnownTextures];

	unsigned myNumKnownTextures;
	TextureInfo myKnownTextures[MaxKnownTextures];

	unsigned UpdateFlags()
	{
		unsigned flags = 0;
		if (myHasFocus)
		{
			flags = myFlagsFromGame | myFlagsFromInput;

			flags &= ~unsigned(CombatModeActive);
			if ((flags & ActiveCheckMask) == ActiveCheckTest)
				flags |= CombatModeActive;
		}
		flags |= Always;
		myFlags = flags;
		return flags;
	}
};

inline UIC_SharedMemory* UIC_GetSharedMemory()
{
	static UIC_SharedMemory* shmem = 0;
	if (shmem)
		return shmem;

	HANDLE hMapFile = CreateFileMapping(
		INVALID_HANDLE_VALUE,						// use paging file
		NULL,										// default security
		PAGE_READWRITE,								// read/write access
		0,											// maximum object size (high-order DWORD)
		sizeof (UIC_SharedMemory),					// maximum object size (low-order DWORD)
		"GuildWars2UICleaner_SharedMemory");		// name of mapping object

	if (!hMapFile)
		return 0;

	void* pBuf = MapViewOfFile(hMapFile,
		FILE_MAP_ALL_ACCESS,
		0,
		0,
		sizeof (UIC_SharedMemory));

	shmem = (UIC_SharedMemory*) pBuf;
	return shmem;
}
